@@ -18,6 +18,7 @@ import ftplib
 import io
 import os
 from datetime import datetime
+from urllib.parse import urljoin, urlparse
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
@@ -27,11 +28,6 @@ from django.utils.deconstruct import deconstructible
 
 from storages.utils import setting
 
-try:
-    from django.utils.six.moves.urllib import parse as urlparse
-except ImportError:
-    from urllib import parse as urlparse
-
 
 class FTPStorageException(Exception):
     pass
@@ -41,13 +37,14 @@ class FTPStorageException(Exception):
 class FTPStorage(Storage):
     """FTP Storage class for Django pluggable storage system."""
 
-    def __init__(self, location=None, base_url=None):
+    def __init__(self, location=None, base_url=None, encoding=None):
         location = location or setting('FTP_STORAGE_LOCATION')
         if location is None:
             raise ImproperlyConfigured("You must set a location at "
                                        "instanciation or at "
                                        " settings.FTP_STORAGE_LOCATION'.")
         self.location = location
+        self.encoding = encoding or setting('FTP_STORAGE_ENCODING') or 'latin-1'
         base_url = base_url or settings.MEDIA_URL
         self._config = self._decode_location(location)
         self._base_url = base_url
@@ -55,7 +52,7 @@ class FTPStorage(Storage):
 
     def _decode_location(self, location):
         """Return splitted configuration data from location."""
-        splitted_url = urlparse.urlparse(location)
+        splitted_url = urlparse(location)
         config = {}
 
         if splitted_url.scheme not in ('ftp', 'aftp'):
@@ -88,6 +85,7 @@ class FTPStorage(Storage):
         # Real reconnect
         if self._connection is None:
             ftp = ftplib.FTP()
+            ftp.encoding = self.encoding
             try:
                 ftp.connect(self._config['host'], self._config['port'])
                 ftp.login(self._config['user'], self._config['passwd'])
@@ -109,7 +107,7 @@ class FTPStorage(Storage):
 
     def _mkremdirs(self, path):
         pwd = self._connection.pwd()
-        path_splitted = os.path.split(path)
+        path_splitted = path.split(os.path.sep)
         for path_part in path_splitted:
             try:
                 self._connection.cwd(path_part)
@@ -246,7 +244,7 @@ class FTPStorage(Storage):
     def url(self, name):
         if self._base_url is None:
             raise ValueError("This file is not accessible via a URL.")
-        return urlparse.urljoin(self._base_url, name).replace('\\', '/')
+        return urljoin(self._base_url, name).replace('\\', '/')
 
 
 class FTPStorageFile(File):
