@@ -3,7 +3,6 @@
 # License: MIT
 #
 # Modeled on the FTP storage by Rafal Jonca <jonca.rafal@gmail.com>
-from __future__ import print_function
 
 import getpass
 import io
@@ -11,47 +10,46 @@ import os
 import posixpath
 import stat
 from datetime import datetime
+from urllib.parse import urljoin
 
 import paramiko
 from django.core.files.base import File
-from django.core.files.storage import Storage
 from django.utils.deconstruct import deconstructible
 
+from storages.base import BaseStorage
 from storages.utils import setting
 
-try:
-    from django.utils.six.moves.urllib import parse as urlparse
-except ImportError:
-    from urllib import parse as urlparse
-
 
 @deconstructible
-class SFTPStorage(Storage):
-
-    def __init__(self, host=None, params=None, interactive=None, file_mode=None,
-                 dir_mode=None, uid=None, gid=None, known_host_file=None,
-                 root_path=None, base_url=None):
-        self._host = host or setting('SFTP_STORAGE_HOST')
-
-        self._params = params or setting('SFTP_STORAGE_PARAMS', {})
-        self._interactive = setting('SFTP_STORAGE_INTERACTIVE', False) \
-            if interactive is None else interactive
-        self._file_mode = setting('SFTP_STORAGE_FILE_MODE') \
-            if file_mode is None else file_mode
-        self._dir_mode = setting('SFTP_STORAGE_DIR_MODE') if \
-            dir_mode is None else dir_mode
-
-        self._uid = setting('SFTP_STORAGE_UID') if uid is None else uid
-        self._gid = setting('SFTP_STORAGE_GID') if gid is None else gid
-        self._known_host_file = setting('SFTP_KNOWN_HOST_FILE') \
-            if known_host_file is None else known_host_file
-
-        self._root_path = setting('SFTP_STORAGE_ROOT', '') \
-            if root_path is None else root_path
-        self._base_url = setting('MEDIA_URL') if base_url is None else base_url
-
+class SFTPStorage(BaseStorage):
+    def __init__(self, **settings):
+        super().__init__(**settings)
+        self._host = self.host
+        self._params = self.params
+        self._interactive = self.interactive
+        self._file_mode = self.file_mode
+        self._dir_mode = self.dir_mode
+        self._uid = self.uid
+        self._gid = self.gid
+        self._known_host_file = self.known_host_file
+        self._root_path = self.root_path
+        self._base_url = self.base_url
         self._sftp = None
 
+    def get_default_settings(self):
+        return {
+            'host': setting('SFTP_STORAGE_HOST'),
+            'params': setting('SFTP_STORAGE_PARAMS', {}),
+            'interactive': setting('SFTP_STORAGE_INTERACTIVE', False),
+            'file_mode': setting('SFTP_STORAGE_FILE_MODE'),
+            'dir_mode': setting('SFTP_STORAGE_DIR_MODE'),
+            'uid': setting('SFTP_STORAGE_UID'),
+            'gid': setting('SFTP_STORAGE_GID'),
+            'known_host_file': setting('SFTP_KNOWN_HOST_FILE'),
+            'root_path': setting('SFTP_STORAGE_ROOT', ''),
+            'base_url': setting('MEDIA_URL'),
+        }
+
     def _connect(self):
         self._ssh = paramiko.SSHClient()
 
@@ -78,8 +76,6 @@ class SFTPStorage(Storage):
                 self._connect()
             else:
                 raise paramiko.AuthenticationException(e)
-        except Exception as e:
-            print(e)
 
         if self._ssh.get_transport():
             self._sftp = self._ssh.open_sftp()
@@ -147,14 +143,14 @@ class SFTPStorage(Storage):
     def delete(self, name):
         try:
             self.sftp.remove(self._remote_path(name))
-        except IOError:
+        except OSError:
             pass
 
     def exists(self, name):
         try:
             self.sftp.stat(self._remote_path(name))
             return True
-        except IOError:
+        except OSError:
             return False
 
     def _isdir_attr(self, item):
@@ -191,7 +187,7 @@ class SFTPStorage(Storage):
     def url(self, name):
         if self._base_url is None:
             raise ValueError("This file is not accessible via a URL.")
-        return urlparse.urljoin(self._base_url, name).replace('\\', '/')
+        return urljoin(self._base_url, name).replace('\\', '/')
 
 
 class SFTPStorageFile(File):
