@@ -1,3 +1,4 @@
+import io
 import os
 import posixpath
 
@@ -5,7 +6,15 @@ from django.conf import settings
 from django.core.exceptions import (
     ImproperlyConfigured, SuspiciousFileOperation,
 )
-from django.utils.encoding import force_text
+from django.utils.encoding import force_bytes
+
+
+def to_bytes(content):
+    """Wrap Django's force_bytes to pass through bytearrays."""
+    if isinstance(content, bytearray):
+        return content
+
+    return force_bytes(content)
 
 
 def setting(name, default=None):
@@ -54,9 +63,9 @@ def safe_join(base, *paths):
     Paths outside the base path indicate a possible security
     sensitive operation.
     """
-    base_path = force_text(base)
+    base_path = base
     base_path = base_path.rstrip('/')
-    paths = [force_text(p) for p in paths]
+    paths = [p for p in paths]
 
     final_path = base_path + '/'
     for path in paths:
@@ -118,3 +127,10 @@ def get_available_overwrite_name(name, max_length):
             'allows sufficient "max_length".' % name
         )
     return os.path.join(dir_name, "{}{}".format(file_root, file_ext))
+
+
+# A buffered reader that does not actually close, workaround for
+# https://github.com/boto/s3transfer/issues/80#issuecomment-562356142
+class NonCloseableBufferedReader(io.BufferedReader):
+    def close(self):
+        self.flush()
