@@ -1,12 +1,10 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 import gzip
 import pickle
 import threading
-import warnings
 from datetime import datetime
-from unittest import skipIf
+from textwrap import dedent
+from unittest import mock, skipIf
+from urllib.parse import urlparse
 
 from botocore.exceptions import ClientError
 from django.conf import settings
@@ -17,17 +15,6 @@ from django.utils.timezone import is_aware, utc
 
 from storages.backends import s3boto3
 
-try:
-    from django.utils.six.moves.urllib import parse as urlparse
-except ImportError:
-    from urllib import parse as urlparse
-
-
-try:
-    from unittest import mock
-except ImportError:  # Python 3.2 and below
-    import mock
-
 
 class S3Boto3TestCase(TestCase):
     def setUp(self):
@@ -35,6 +22,12 @@ class S3Boto3TestCase(TestCase):
         self.storage._connections.connection = mock.MagicMock()
 
 
+def assert_called_upload(mock_uploadobj, content, extra):
+    assert mock_uploadobj.call_count == 1
+    assert mock_uploadobj.call_args[0][0].raw == content
+    assert mock_uploadobj.call_args[1] == extra
+
+
 class S3Boto3StorageTests(S3Boto3TestCase):
 
     def test_clean_name(self):
@@ -120,15 +113,14 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         self.storage.bucket.Object.assert_called_once_with(name)
 
         obj = self.storage.bucket.Object.return_value
-        obj.upload_fileobj.assert_called_with(
-            content,
-            ExtraArgs={
+        extra = {
+            'ExtraArgs': {
                 'ContentType': 'text/plain',
-                'ACL': self.storage.default_acl,
             }
-        )
+        }
+        assert_called_upload(obj.upload_fileobj, content, extra)
 
-    def test_storage_save_with_acl(self):
+    def test_storage_save_with_default_acl(self):
         """
         Test saving a file with user defined ACL.
         """
@@ -139,13 +131,33 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         self.storage.bucket.Object.assert_called_once_with(name)
 
         obj = self.storage.bucket.Object.return_value
-        obj.upload_fileobj.assert_called_with(
-            content,
-            ExtraArgs={
+        extra = {
+            'ExtraArgs': {
                 'ContentType': 'text/plain',
                 'ACL': 'private',
             }
-        )
+        }
+        assert_called_upload(obj.upload_fileobj, content, extra)
+
+    def test_storage_object_parameters_not_overwritten_by_default(self):
+        """
+        Test saving a file with user defined ACL.
+        """
+        name = 'test_storage_save.txt'
+        content = ContentFile('new content')
+        self.storage.default_acl = 'public-read'
+        self.storage.object_parameters = {'ACL': 'private'}
+        self.storage.save(name, content)
+        self.storage.bucket.Object.assert_called_once_with(name)
+
+        obj = self.storage.bucket.Object.return_value
+        extra = {
+            'ExtraArgs': {
+                'ContentType': 'text/plain',
+                'ACL': 'private',
+            }
+        }
+        assert_called_upload(obj.upload_fileobj, content, extra)
 
     def test_content_type(self):
         """
@@ -158,13 +170,12 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         self.storage.bucket.Object.assert_called_once_with(name)
 
         obj = self.storage.bucket.Object.return_value
-        obj.upload_fileobj.assert_called_with(
-            content,
-            ExtraArgs={
+        extra = {
+            'ExtraArgs': {
                 'ContentType': 'image/jpeg',
-                'ACL': self.storage.default_acl,
             }
-        )
+        }
+        assert_called_upload(obj.upload_fileobj, content, extra)
 
     def test_storage_save_gzipped(self):
         """
@@ -174,14 +185,13 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         content = ContentFile("I am gzip'd")
         self.storage.save(name, content)
         obj = self.storage.bucket.Object.return_value
-        obj.upload_fileobj.assert_called_with(
-            content,
-            ExtraArgs={
+        extra = {
+            'ExtraArgs': {
                 'ContentType': 'application/octet-stream',
                 'ContentEncoding': 'gzip',
-                'ACL': self.storage.default_acl,
             }
-        )
+        }
+        assert_called_upload(obj.upload_fileobj, content, extra)
 
     def test_storage_save_gzip(self):
         """
@@ -197,7 +207,6 @@ class S3Boto3StorageTests(S3Boto3TestCase):
             ExtraArgs={
                 'ContentType': 'text/css',
                 'ContentEncoding': 'gzip',
-                'ACL': self.storage.default_acl,
             }
         )
         args, kwargs = obj.upload_fileobj.call_args
@@ -225,7 +234,6 @@ class S3Boto3StorageTests(S3Boto3TestCase):
             ExtraArgs={
                 'ContentType': 'text/css',
                 'ContentEncoding': 'gzip',
-                'ACL': self.storage.default_acl,
             }
         )
         args, kwargs = obj.upload_fileobj.call_args
@@ -242,6 +250,14 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         content = self.storage._compress_content(content)
         self.assertTrue(len(content.read()) > 0)
 
+    def test_storage_open_read_string(self):
+        """
+        Test opening a file in "r" mode (ie reading as string, not bytes)
+        """
+        name = 'test_open_read_string.txt'
+        content_str = self.storage.open(name, "r").read()
+        self.assertEqual(content_str, "")
+
     def test_storage_open_write(self):
         """
         Test opening a file in write mode
@@ -250,9 +266,11 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         content = 'new content'
 
         # Set the encryption flag used for multipart uploads
-        self.storage.encryption = True
-        self.storage.reduced_redundancy = True
-        self.storage.default_acl = 'public-read'
+        self.storage.object_parameters = {
+            'ServerSideEncryption': 'AES256',
+            'StorageClass': 'REDUCED_REDUNDANCY',
+            'ACL': 'public-read',
+        }
 
         file = self.storage.open(name, 'w')
         self.storage.bucket.Object.assert_called_with(name)
@@ -274,10 +292,21 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         file.close()
         multipart.Part.assert_called_with(1)
         part = multipart.Part.return_value
-        part.upload.assert_called_with(Body=content.encode('utf-8'))
+        part.upload.assert_called_with(Body=content.encode())
         multipart.complete.assert_called_once_with(
             MultipartUpload={'Parts': [{'ETag': '123', 'PartNumber': 1}]})
 
+    def test_write_bytearray(self):
+        """Test that bytearray write exactly (no extra "bytearray" from stringify)."""
+        name = "saved_file.bin"
+        content = bytearray(b"content")
+        file = self.storage.open(name, "wb")
+        obj = self.storage.bucket.Object.return_value
+        # Set the name of the mock object
+        obj.key = name
+        bytes_written = file.write(content)
+        self.assertEqual(len(content), bytes_written)
+
     def test_storage_open_no_write(self):
         """
         Test opening file in write mode and closing without writing.
@@ -287,9 +316,10 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         name = 'test_open_no_write.txt'
 
         # Set the encryption flag used for puts
-        self.storage.encryption = True
-        self.storage.reduced_redundancy = True
-        self.storage.default_acl = 'public-read'
+        self.storage.object_parameters = {
+            'ServerSideEncryption': 'AES256',
+            'StorageClass': 'REDUCED_REDUNDANCY',
+        }
 
         file = self.storage.open(name, 'w')
         self.storage.bucket.Object.assert_called_with(name)
@@ -306,7 +336,6 @@ class S3Boto3StorageTests(S3Boto3TestCase):
 
         obj.load.assert_called_once_with()
         obj.put.assert_called_once_with(
-            ACL='public-read',
             Body=b"",
             ContentType='text/plain',
             ServerSideEncryption='AES256',
@@ -320,9 +349,10 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         name = 'test_open_no_overwrite_existing.txt'
 
         # Set the encryption flag used for puts
-        self.storage.encryption = True
-        self.storage.reduced_redundancy = True
-        self.storage.default_acl = 'public-read'
+        self.storage.object_parameters = {
+            'ServerSideEncryption': 'AES256',
+            'StorageClass': 'REDUCED_REDUNDANCY',
+        }
 
         file = self.storage.open(name, 'w')
         self.storage.bucket.Object.assert_called_with(name)
@@ -344,9 +374,10 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         name = 'test_open_for_writïng_beyond_buffer_size.txt'
 
         # Set the encryption flag used for multipart uploads
-        self.storage.encryption = True
-        self.storage.reduced_redundancy = True
-        self.storage.default_acl = 'public-read'
+        self.storage.object_parameters = {
+            'ServerSideEncryption': 'AES256',
+            'StorageClass': 'REDUCED_REDUNDANCY',
+        }
 
         file = self.storage.open(name, 'w')
         self.storage.bucket.Object.assert_called_with(name)
@@ -357,7 +388,6 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         # Initiate the multipart upload
         file.write('')
         obj.initiate_multipart_upload.assert_called_with(
-            ACL='public-read',
             ContentType='text/plain',
             ServerSideEncryption='AES256',
             StorageClass='REDUCED_REDUNDANCY'
@@ -388,7 +418,7 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         )
         part = multipart.Part.return_value
         uploaded_content = ''.join(
-            args_list[1]['Body'].decode('utf-8')
+            args_list[1]['Body'].decode()
             for args_list in part.upload.call_args_list
         )
         self.assertEqual(uploaded_content, written_content)
@@ -399,41 +429,6 @@ class S3Boto3StorageTests(S3Boto3TestCase):
             ]}
         )
 
-    def test_auto_creating_bucket(self):
-        self.storage.auto_create_bucket = True
-        Bucket = mock.MagicMock()
-        self.storage._connections.connection.Bucket.return_value = Bucket
-        self.storage._connections.connection.meta.client.meta.region_name = 'sa-east-1'
-
-        Bucket.meta.client.head_bucket.side_effect = ClientError({'Error': {},
-                                                                  'ResponseMetadata': {'HTTPStatusCode': 404}},
-                                                                 'head_bucket')
-        self.storage._get_or_create_bucket('testbucketname')
-        Bucket.create.assert_called_once_with(
-            ACL='public-read',
-            CreateBucketConfiguration={
-                'LocationConstraint': 'sa-east-1',
-            }
-        )
-
-    def test_auto_creating_bucket_with_acl(self):
-        self.storage.auto_create_bucket = True
-        self.storage.bucket_acl = 'public-read'
-        Bucket = mock.MagicMock()
-        self.storage._connections.connection.Bucket.return_value = Bucket
-        self.storage._connections.connection.meta.client.meta.region_name = 'sa-east-1'
-
-        Bucket.meta.client.head_bucket.side_effect = ClientError({'Error': {},
-                                                                  'ResponseMetadata': {'HTTPStatusCode': 404}},
-                                                                 'head_bucket')
-        self.storage._get_or_create_bucket('testbucketname')
-        Bucket.create.assert_called_once_with(
-            ACL='public-read',
-            CreateBucketConfiguration={
-                'LocationConstraint': 'sa-east-1',
-            }
-        )
-
     def test_storage_exists(self):
         self.assertTrue(self.storage.exists("file.txt"))
         self.storage.connection.meta.client.head_object.assert_called_with(
@@ -452,11 +447,6 @@ class S3Boto3StorageTests(S3Boto3TestCase):
             Key='file.txt',
         )
 
-    def test_storage_exists_doesnt_create_bucket(self):
-        with mock.patch.object(self.storage, '_get_or_create_bucket') as method:
-            self.storage.exists('file.txt')
-            self.assertFalse(method.called)
-
     def test_storage_delete(self):
         self.storage.delete("path/to/file.txt")
         self.storage.bucket.Object.assert_called_with('path/to/file.txt')
@@ -557,7 +547,8 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         self.storage.bucket.meta.client.generate_presigned_url.assert_called_with(
             'get_object',
             Params={'Bucket': self.storage.bucket.name, 'Key': name},
-            ExpiresIn=self.storage.querystring_expire
+            ExpiresIn=self.storage.querystring_expire,
+            HttpMethod=None,
         )
 
         custom_expire = 123
@@ -566,14 +557,64 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         self.storage.bucket.meta.client.generate_presigned_url.assert_called_with(
             'get_object',
             Params={'Bucket': self.storage.bucket.name, 'Key': name},
-            ExpiresIn=custom_expire
+            ExpiresIn=custom_expire,
+            HttpMethod=None,
         )
 
+        custom_method = 'HEAD'
+
+        self.assertEqual(self.storage.url(name, http_method=custom_method), url)
+        self.storage.bucket.meta.client.generate_presigned_url.assert_called_with(
+            'get_object',
+            Params={'Bucket': self.storage.bucket.name, 'Key': name},
+            ExpiresIn=self.storage.querystring_expire,
+            HttpMethod=custom_method,
+        )
+
+    def test_storage_url_custom_domain_signed_urls(self):
+        key_id = 'test-key'
+        filename = 'file.txt'
+        pem = dedent(
+            '''\
+            -----BEGIN RSA PRIVATE KEY-----
+            MIICWwIBAAKBgQCXVuwcMk+JmVSKuQ1K4dZx4Z1dEcRQgTlqvhAyljIpttXlZh2/
+            fD3GkJCiqfwEmo+cdNK/LFzRj/CX8Wz1z1lH2USONpG6sAkotkatCbejiItDu5y6
+            janGJHfuWXu6B/o9gwZylU1gIsePY3lLNk+r9QhXUO4jXw6zLJftVwKPhQIDAQAB
+            AoGAbpkRV9HUmoQ5al+uPSkp5HOy4s8XHpYxdbaMc8ubwSxiyJCF8OhE5RXE/Xso
+            N90UUox1b0xmUKfWddPzgvgTD/Ub7D6Ukf+nVWDX60tWgNxICAUHptGL3tWweaAy
+            H+0+vZ0TzvTt9r00vW0FzO7F8X9/Rs1ntDRLtF3RCCxdq0kCQQDHFu+t811lCvy/
+            67rMEKGvNsNNSTrzOrNr3PqUrCnOrzKazjFVjsKv5VzI/U+rXGYKWJsMpuCFiHZ3
+            DILUC09TAkEAwpm2S6MN6pzn9eY6pmhOxZ+GQGGRUkKZfC1GDxaRSRb8sKTjptYw
+            WSemJSxiDzdj3Po2hF0lbhkpJgUq6xnCxwJAZgHHfn5CLSJrDD7Q7/vZi/foK3JJ
+            BRTfl3Wa4pAvv5meuRjKyEakVBGV79lyd5+ZHNX3Y40hXunjoO3FHrZIxwJAdRzu
+            waxahrRxQOKSr20c4wAzWnGddIUSO9I/VHs/al5EKsbBHrnOlQkwizSfuwqZtfZ7
+            csNf8FeCFRiNELoLJwJAZxWBE2+8J9VW9AQ0SE7j4FyM/B8FvRhF5PLAAsw/OxHO
+            SxiFP7Ptdac1tm5H5zOqaqSHWphI19HNNilXKmxuCA==
+            -----END RSA PRIVATE KEY-----'''
+        ).encode('ascii')
+
+        url = 'https://mock.cloudfront.net/file.txt'
+        signed_url = url + '?Expires=3600&Signature=DbqVgh3FHtttQxof214tSAVE8Nqn3Q4Ii7eR3iykbOqAPbV89HC3EB~0CWxarpLNtbfosS5LxiP5EutriM7E8uR4Gm~UVY-PFUjPcwqdnmAiKJF0EVs7koJcMR8MKDStuWfFKVUPJ8H7ORYTOrixyHBV2NOrpI6SN5UX6ctNM50_&Key-Pair-Id=test-key'  # noqa
+
+        self.storage.custom_domain = "mock.cloudfront.net"
+
+        for pem_to_signer in (
+                s3boto3._use_cryptography_signer(),
+                s3boto3._use_rsa_signer()):
+            self.storage.cloudfront_signer = pem_to_signer(key_id, pem)
+            self.storage.querystring_auth = False
+            self.assertEqual(self.storage.url(filename), url)
+
+            self.storage.querystring_auth = True
+            with mock.patch('storages.backends.s3boto3.datetime') as mock_datetime:
+                mock_datetime.utcnow.return_value = datetime.utcfromtimestamp(0)
+                self.assertEqual(self.storage.url(filename), signed_url)
+
     def test_generated_url_is_encoded(self):
         self.storage.custom_domain = "mock.cloudfront.net"
         filename = "whacky & filename.mp4"
         url = self.storage.url(filename)
-        parsed_url = urlparse.urlparse(url)
+        parsed_url = urlparse(url)
         self.assertEqual(parsed_url.path,
                          "/whacky%20%26%20filename.mp4")
         self.assertFalse(self.storage.bucket.meta.client.generate_presigned_url.called)
@@ -587,7 +628,7 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         self.storage.bucket.Object.assert_called_once_with(name)
 
         url = self.storage.url(name)
-        parsed_url = urlparse.urlparse(url)
+        parsed_url = urlparse(url)
         self.assertEqual(parsed_url.path, "/%C3%A3l%C3%B6h%C3%A2.jpg")
 
     def test_strip_signing_parameters(self):
@@ -620,61 +661,29 @@ class S3Boto3StorageTests(S3Boto3TestCase):
         with self.assertRaises(ImproperlyConfigured, msg=msg):
             s3boto3.S3Boto3Storage(location='/')
 
-    def test_deprecated_acl(self):
-        with override_settings(AWS_DEFAULT_ACL=None), warnings.catch_warnings(record=True) as w:
-            s3boto3.S3Boto3Storage(acl='private')
-        assert len(w) == 1
-        assert issubclass(w[-1].category, DeprecationWarning)
-        message = (
-            "The acl argument of S3Boto3Storage is deprecated. Use argument "
-            "default_acl or setting AWS_DEFAULT_ACL instead. The acl argument "
-            "will be removed in version 1.10."
-        )
-        assert str(w[-1].message) == message
+    def test_override_settings(self):
+        with override_settings(AWS_LOCATION='foo1'):
+            storage = s3boto3.S3Boto3Storage()
+            self.assertEqual(storage.location, 'foo1')
+        with override_settings(AWS_LOCATION='foo2'):
+            storage = s3boto3.S3Boto3Storage()
+            self.assertEqual(storage.location, 'foo2')
 
-    def test_deprecated_bucket(self):
-        with override_settings(AWS_DEFAULT_ACL=None), warnings.catch_warnings(record=True) as w:
-            s3boto3.S3Boto3Storage(bucket='django')
-        assert len(w) == 1
-        assert issubclass(w[-1].category, DeprecationWarning)
-        message = (
-            "The bucket argument of S3Boto3Storage is deprecated. Use argument "
-            "bucket_name or setting AWS_STORAGE_BUCKET_NAME instead. The bucket "
-            "argument will be removed in version 1.10."
-        )
-        assert str(w[-1].message) == message
+    def test_override_class_variable(self):
+        class MyStorage1(s3boto3.S3Boto3Storage):
+            location = 'foo1'
 
-    def test_deprecated_default_acl(self):
-        with warnings.catch_warnings(record=True) as w:
-            s3boto3.S3Boto3Storage()
-        assert len(w) == 1
-        message = (
-            "The default behavior of S3Boto3Storage is insecure and will change "
-            "in django-storages 1.10. By default files and new buckets are saved "
-            "with an ACL of 'public-read' (globally publicly readable). Version 1.10 will "
-            "default to using the bucket's ACL. To opt into the new behavior set "
-            "AWS_DEFAULT_ACL = None, otherwise to silence this warning explicitly "
-            "set AWS_DEFAULT_ACL."
-        )
-        assert str(w[-1].message) == message
+        storage = MyStorage1()
+        self.assertEqual(storage.location, 'foo1')
 
-    def test_deprecated_autocreate_bucket(self):
-        with override_settings(AWS_DEFAULT_ACL=None), warnings.catch_warnings(record=True) as w:
-            s3boto3.S3Boto3Storage(auto_create_bucket=True)
-        assert len(w) == 1
-        assert issubclass(w[-1].category, DeprecationWarning)
-        message = (
-            "Automatic bucket creation will be removed in version 1.10. It encourages "
-            "using overly broad credentials with this library. Either create it before "
-            "manually or use one of a myriad of automatic configuration management tools. "
-            "Unset AWS_AUTO_CREATE_BUCKET (it defaults to False) to silence this warning."
-        )
-        assert str(w[-1].message) == message
+        class MyStorage2(s3boto3.S3Boto3Storage):
+            location = 'foo2'
 
-    def test_deprecated_default_acl_override_class_variable(self):
-        class MyStorage(s3boto3.S3Boto3Storage):
-            default_acl = "private"
+        storage = MyStorage2()
+        self.assertEqual(storage.location, 'foo2')
 
-        with warnings.catch_warnings(record=True) as w:
-            MyStorage()
-        assert len(w) == 0
+    def test_override_init_argument(self):
+        storage = s3boto3.S3Boto3Storage(location='foo1')
+        self.assertEqual(storage.location, 'foo1')
+        storage = s3boto3.S3Boto3Storage(location='foo2')
+        self.assertEqual(storage.location, 'foo2')
